package com.hexin.android.component.listview

import android.view.View
import com.hexin.plat.android.HexinApplication
import com.hexin.util.UIFrameworkConfig

//import com.hxcommonlibrary.utils.UIFrameworkConfig

object ListComponentUtils {

    /**
     * 计算可以滑动的宽度
     * @param totalWidth 屏幕上显示的总宽度，基本上情况都是屏幕宽度
     * @param view 需要计算的视图
     */
    fun calculateAvailableToScrollWidth(totalWidth: Int, view: View?): Int {
        if (view !is SlidingItem) return 0
        val slidingView = view.getSlidingView() ?: return 0

        val sizeArray = calculateViewSize(slidingView)

        val availableToScrollWidth = sizeArray[0] - (totalWidth - view.getFixedViewWidth())
        return if (availableToScrollWidth > 0) {
            availableToScrollWidth
        } else {
            0
        }
    }

    fun calculateViewSize(view: View?): IntArray {
        val sizeArray = IntArray(2)
        if (view == null) {
            return sizeArray
        }

        val widthMeasureSpec = View.MeasureSpec.makeMeasureSpec((1 shl 30) - 1, View.MeasureSpec.AT_MOST)
        val heightMeasureSpec = View.MeasureSpec.makeMeasureSpec((1 shl 30) - 1, View.MeasureSpec.AT_MOST)
        view.measure(widthMeasureSpec, heightMeasureSpec)

        sizeArray[0] = view.measuredWidth
        sizeArray[1] = view.measuredHeight
        return sizeArray
    }

    private const val TRIPLE_COLUMNS = 3
    private const val QUADRUPLE_COLUMNS = 4
    var listColumnCount = TRIPLE_COLUMNS //用于确定listColumn的数量，便于处理小箭头的逻辑

    /**
     * 计算当前屏幕下，每一项“可滑动项”应设置的宽度
     * @param fixWidth 固定项的宽度
     * @param columnWidth 可滑动项的原始宽度
     */
    fun calculateListColumnWidth(fixWidth: Int, columnWidth: Int): Int {
        var calWidth = columnWidth

        val windowWidth = UIFrameworkConfig.getWindowWidth(HexinApplication.getHxApplication())
        if (windowWidth > 0) {
            if (windowWidth > fixWidth.plus(calWidth.times(TRIPLE_COLUMNS))
                    && windowWidth < fixWidth.plus(calWidth.times(QUADRUPLE_COLUMNS))) {
                calWidth = if ((windowWidth - fixWidth) % TRIPLE_COLUMNS == 0) {
                    (windowWidth - fixWidth) / TRIPLE_COLUMNS
                } else {
                    (windowWidth - fixWidth) / TRIPLE_COLUMNS + 1
                }
                listColumnCount = TRIPLE_COLUMNS
            } else if (windowWidth >= fixWidth + calWidth * QUADRUPLE_COLUMNS) {
                calWidth = if ((windowWidth - fixWidth) % QUADRUPLE_COLUMNS == 0) {
                    (windowWidth - fixWidth) / QUADRUPLE_COLUMNS
                } else {
                    (windowWidth - fixWidth) / QUADRUPLE_COLUMNS + 1
                }
                listColumnCount = QUADRUPLE_COLUMNS
            }
        }
        return calWidth
    }

}
