package com.hexin.android.component.listview

import android.content.Context
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import android.view.MotionEvent

class SlidingRecyclerView(context: Context) : RecyclerView(context), SlidingInfoPicker {
    var listNestedScrollView: ListNestedScrollView? = null
    var onSlidingListeners = ArrayList<OnSlidingListener>()
    var onScrollStateChangedListener: OnScrollStateChangedListener? = null
    var slidingRecyclerViewClickListener: OnSlidingRecyclerViewClickListener? = null
    var parentNeedScrollHorizontal = false  //recycleview的外层布局是否需要横向滑动，如果需要，则对横向滑动冲突进行处理

    private var callback = (object : ViewSlideHelper.Callback() {
        override fun getScrollState(): Int {
            return this@SlidingRecyclerView.scrollState
        }

        override fun getParentNeedScrollHorizontal(): Boolean {
            return this@SlidingRecyclerView.parentNeedScrollHorizontal
        }

        override fun notifyOnSliding(scrolledX: Int, availableToScrollWidth: Int) {
            toNotifyOnSliding(scrolledX, availableToScrollWidth)
        }

        override fun onSlidingRecyclerViewClick(x: Float, y: Float) {
            super.onSlidingRecyclerViewClick(x, y)
            this@SlidingRecyclerView.slidingRecyclerViewClickListener?.onSlidingRecyclerViewClick(x, y)
        }
    })
    private var viewSlideHelper = ViewSlideHelper.create(this, callback)

    override fun onTouchEvent(e: MotionEvent?): Boolean {
        val process = viewSlideHelper?.processTouchEvent(e)
        return process!! || super.onTouchEvent(e)
    }

    override fun onInterceptTouchEvent(e: MotionEvent?): Boolean {
        isSlideToBottom()
        val process = viewSlideHelper?.processInterceptTouchEvent(e)
        return process!! || super.onInterceptTouchEvent(e)
    }

    override fun computeScroll() {
        viewSlideHelper?.computeScroll()
        super.computeScroll()
    }

    private fun toNotifyOnSliding(scrolledX: Int, availableToScrollWidth: Int) {
        for (listener in onSlidingListeners) {
            listener.onSliding(scrolledX, availableToScrollWidth)
        }
    }

    fun isSlideToBottom() {
        if (computeVerticalScrollExtent() + computeVerticalScrollOffset() >= computeVerticalScrollRange()) {
            stopScroll()
        }
    }

    override fun onScrollStateChanged(state: Int) {
        super.onScrollStateChanged(state)
        if (state == RecyclerView.SCROLL_STATE_IDLE) {
            notifyFlingFinished()
        }
        onScrollStateChangedListener?.onScrollStateChanged(state)
    }

    fun scrollToRight() {
        viewSlideHelper?.stopHorizontalFling()
        viewSlideHelper?.scrollToRight()
    }

    fun scrollToLeft() {
        viewSlideHelper?.stopHorizontalFling()
        viewSlideHelper?.scrollToLeft()
    }

    /**
     * availableWidth改变后，调整超出availableWidth的滑动，
     */
    fun scrollToRightIfScrolledXOverAvailableWidth() {
        viewSlideHelper?.scrollToRightIfScrolledXOverAvailableWidth()
    }

    /**
     * 消耗剩余的速度
     */
    fun consumeRemaindVelocity(velocityY: Int) {
        fling(0, velocityY)
    }

    fun stopHorizontalFling() {
        viewSlideHelper?.stopHorizontalFling()
    }

    override fun getScrolledX(): Int? {
        return viewSlideHelper?.scrolledX
    }

    fun setScrolledX(x: Int) {
        viewSlideHelper?.setInitScrolledX(x)
    }

    private fun notifyFlingFinished() {
        val manager = layoutManager as LinearLayoutManager
        if (manager.findFirstVisibleItemPosition() == 0) {
            listNestedScrollView?.consumeRemaindVelocity()
        }
    }
}
